@@ -30,15 +30,16 @@
 #include "empathy-call-handler.h"
 #include "empathy-dispatcher.h"
 #include "empathy-marshal.h"
+#include "empathy-utils.h"
 
 G_DEFINE_TYPE(EmpathyCallHandler, empathy_call_handler, G_TYPE_OBJECT)
 
 /* signal enum */
-enum
-{
+enum {
   CONFERENCE_ADDED,
   SRC_PAD_ADDED,
   SINK_PAD_ADDED,
+  CLOSED,
   LAST_SIGNAL
 };
 
@@ -51,32 +52,67 @@ enum {
 };
 
 /* private structure */
-typedef struct _EmpathyCallHandlerPriv EmpathyCallHandlerPriv;
 
-struct _EmpathyCallHandlerPriv
-{
+typedef struct {
   gboolean dispose_has_run;
   EmpathyTpCall *call;
   EmpathyContact *contact;
   TfChannel *tfchannel;
   GstBus *bus;
-};
+} EmpathyCallHandlerPriv;
 
-#define GET_PRIV(o) \
-  (G_TYPE_INSTANCE_GET_PRIVATE ((o), EMPATHY_TYPE_CALL_HANDLER,\
-    EmpathyCallHandlerPriv))
+#define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyCallHandler)
+
+static void
+empathy_call_handler_dispose (GObject *object)
+{
+  EmpathyCallHandlerPriv *priv = GET_PRIV (object);
+
+  if (priv->dispose_has_run)
+    return;
+
+  priv->dispose_has_run = TRUE;
+
+  if (priv->contact != NULL)
+    g_object_unref (priv->contact);
+
+  priv->contact = NULL;
+
+  if (priv->tfchannel != NULL)
+    g_object_unref (priv->tfchannel);
+
+  priv->tfchannel = NULL;
+
+  if (priv->call != NULL)
+    {
+      empathy_tp_call_close (priv->call);
+      g_object_unref (priv->call);
+    }
+
+  priv->call = NULL;
+
+  /* release any references held by the object here */
+  if (G_OBJECT_CLASS (empathy_call_handler_parent_class)->dispose)
+    G_OBJECT_CLASS (empathy_call_handler_parent_class)->dispose (object);
+}
+
+static void
+empathy_call_handler_finalize (GObject *object)
+{
+  /* free any data held directly by the object here */
+  if (G_OBJECT_CLASS (empathy_call_handler_parent_class)->finalize)
+    G_OBJECT_CLASS (empathy_call_handler_parent_class)->finalize (object);
+}
 
 static void
 empathy_call_handler_init (EmpathyCallHandler *obj)
 {
-  //EmpathyCallHandlerPriv *priv = GET_PRIV (obj);
+  EmpathyCallHandlerPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (obj,
+    EMPATHY_TYPE_CALL_HANDLER, EmpathyCallHandlerPriv);
 
-  /* allocate any data required by the object here */
+  obj->priv = priv;
 }
 
-static void empathy_call_handler_dispose (GObject *object);
-static void empathy_call_handler_finalize (GObject *object);
-
 static void
 empathy_call_handler_set_property (GObject *object,
   guint property_id, const GValue *value, GParamSpec *pspec)
@@ -173,51 +209,13 @@ empathy_call_handler_class_init (EmpathyCallHandlerClass *klass)
       _empathy_marshal_VOID__OBJECT_UINT,
       G_TYPE_NONE,
       2, GST_TYPE_PAD, G_TYPE_UINT);
-}
 
-void
-empathy_call_handler_dispose (GObject *object)
-{
-  EmpathyCallHandler *self = EMPATHY_CALL_HANDLER (object);
-  EmpathyCallHandlerPriv *priv = GET_PRIV (self);
-
-  if (priv->dispose_has_run)
-    return;
-
-  priv->dispose_has_run = TRUE;
-
-  if (priv->contact != NULL)
-    g_object_unref (priv->contact);
-
-  priv->contact = NULL;
-
-  if (priv->tfchannel != NULL)
-    g_object_unref (priv->tfchannel);
-
-  priv->tfchannel = NULL;
-
-  if (priv->call != NULL)
-    {
-      empathy_tp_call_close (priv->call);
-      g_object_unref (priv->call);
-    }
-
-  priv->call = NULL;
-
-  /* release any references held by the object here */
-  if (G_OBJECT_CLASS (empathy_call_handler_parent_class)->dispose)
-    G_OBJECT_CLASS (empathy_call_handler_parent_class)->dispose (object);
-}
-
-void
-empathy_call_handler_finalize (GObject *object)
-{
-  //EmpathyCallHandler *self = EMPATHY_CALL_HANDLER (object);
-  //EmpathyCallHandlerPriv *priv = GET_PRIV (self);
-
-  /* free any data held directly by the object here */
-
-  G_OBJECT_CLASS (empathy_call_handler_parent_class)->finalize (object);
+  signals[CLOSED] =
+    g_signal_new ("closed", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL,
+      g_cclosure_marshal_VOID__VOID,
+      G_TYPE_NONE,
+      0);
 }
 
 EmpathyCallHandler *
@@ -303,6 +301,13 @@ empathy_call_handler_tf_channel_stream_created_cb (TfChannel *tfchannel,
   gst_object_unref (spad);
 }
 
+static void
+empathy_call_handler_tf_channel_closed_cb (TfChannel *tfchannel,
+  EmpathyCallHandler *handler)
+{
+  g_signal_emit (G_OBJECT (handler), signals[CLOSED], 0);
+}
+
 static void
 empathy_call_handler_start_tpfs (EmpathyCallHandler *self)
 {
@@ -320,6 +325,8 @@ empathy_call_handler_start_tpfs (EmpathyCallHandler *self)
       G_CALLBACK (empathy_call_handler_tf_channel_session_created_cb), self);
   g_signal_connect (priv->tfchannel, "stream-created",
       G_CALLBACK (empathy_call_handler_tf_channel_stream_created_cb), self);
+  g_signal_connect (priv->tfchannel, "closed",
+      G_CALLBACK (empathy_call_handler_tf_channel_closed_cb), self);
 
   g_object_unref (channel);
 }
@@ -400,7 +407,7 @@ empathy_call_handler_start_call (EmpathyCallHandler *handler)
   if (priv->call == NULL)
     {
       empathy_contact_call_when_ready (priv->contact,
-        EMPATHY_CONTACT_READY_ID,
+        EMPATHY_CONTACT_READY_HANDLE,
         empathy_call_handler_contact_ready_cb, NULL, NULL, G_OBJECT (handler));
     }
   else
