@@ -33,6 +33,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <glade/glade.h>
 #include <glib/gi18n.h>
+#include <libnotify/notification.h>
 
 #include <telepathy-glib/util.h>
 #include <libmissioncontrol/mission-control.h>
@@ -54,6 +55,7 @@
 
 #include "empathy-chat-window.h"
 #include "empathy-about-dialog.h"
+#include "empathy-misc.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_CHAT
 #include <libempathy/empathy-debug.h>
@@ -70,6 +72,7 @@ typedef struct {
 	EmpathyChatroomManager *chatroom_manager;
 	GtkWidget   *dialog;
 	GtkWidget   *notebook;
+	NotifyNotification *notification;
 
 	/* Menu items. */
 	GtkWidget   *menu_conv_clear;
@@ -325,7 +328,10 @@ chat_window_update (EmpathyChatWindow *window)
 			remote_contact = empathy_chat_get_remote_contact (priv->current_chat);
 			icon = empathy_pixbuf_avatar_from_contact_scaled (remote_contact, 0, 0);
 			gtk_window_set_icon (GTK_WINDOW (priv->dialog), icon);
-			g_object_unref (icon);
+
+			if (icon != NULL) {
+				g_object_unref (icon);
+			}
 		} else {
 			gtk_window_set_icon_name (GTK_WINDOW (priv->dialog), NULL);
 		}
@@ -830,6 +836,86 @@ chat_window_set_urgency_hint (EmpathyChatWindow *window,
 	gtk_window_set_urgency_hint (GTK_WINDOW (priv->dialog), urgent);
 }
 
+typedef struct {
+	EmpathyChatWindow *window;
+	EmpathyChat *chat;
+} NotificationData;
+
+static void
+chat_window_notification_closed_cb (NotifyNotification *notify,
+				    NotificationData *cb_data)
+{
+	EmpathyNotificationClosedReason reason = 0;
+	EmpathyChatWindowPriv *priv = GET_PRIV (cb_data->window);
+
+#ifdef notify_notification_get_closed_reason
+	reason = notify_notification_get_closed_reason (notify);
+#endif
+	if (reason == EMPATHY_NOTIFICATION_CLOSED_DISMISSED) {
+		empathy_chat_window_present_chat (cb_data->chat);
+	}
+
+	g_object_unref (notify);
+	priv->notification = NULL;
+	g_object_unref (cb_data->chat);
+	g_slice_free (NotificationData, cb_data);
+}
+
+static void
+chat_window_show_or_update_notification (EmpathyChatWindow *window,
+					 EmpathyMessage *message,
+					 EmpathyChat *chat)
+{
+	EmpathyContact *sender;
+	char *header, *escaped;
+	const char *body;
+	GdkPixbuf *pixbuf;
+	NotificationData *cb_data;
+	EmpathyChatWindowPriv *priv = GET_PRIV (window);
+	gboolean res;
+
+	if (!empathy_notification_is_enabled ()) {
+		return;
+	} else {
+		empathy_conf_get_bool (empathy_conf_get (),
+				       EMPATHY_PREFS_NOTIFICATIONS_FOCUS, &res);
+		if (!res) {
+			return;
+		}
+	}
+
+	cb_data = g_slice_new0 (NotificationData);
+	cb_data->chat = g_object_ref (chat);
+	cb_data->window = window;
+
+	sender = empathy_message_get_sender (message);
+	header = g_strdup_printf (_("New message from %s"),
+				  empathy_contact_get_name (sender));
+	body = empathy_message_get_body (message);
+	escaped = g_markup_escape_text (body, -1);
+
+	pixbuf = empathy_misc_get_pixbuf_for_notification (sender, EMPATHY_IMAGE_NEW_MESSAGE);
+
+	if (priv->notification != NULL) {
+		notify_notification_update (priv->notification,
+					    header, escaped, NULL);
+		notify_notification_set_icon_from_pixbuf (priv->notification, pixbuf);
+	} else {
+		priv->notification = notify_notification_new (header, escaped, NULL, NULL);
+		notify_notification_set_timeout (priv->notification, NOTIFY_EXPIRES_DEFAULT);
+		notify_notification_set_icon_from_pixbuf (priv->notification, pixbuf);
+
+		g_signal_connect (priv->notification, "closed",
+				  G_CALLBACK (chat_window_notification_closed_cb), cb_data);
+	}
+
+	notify_notification_show (priv->notification, NULL);
+
+	g_object_unref (pixbuf);
+	g_free (header);
+	g_free (escaped);
+}
+
 static void
 chat_window_new_message_cb (EmpathyChat       *chat,
 			    EmpathyMessage    *message,
@@ -838,11 +924,35 @@ chat_window_new_message_cb (EmpathyChat       *chat,
 	EmpathyChatWindowPriv *priv;
 	gboolean              has_focus;
 	gboolean              needs_urgency;
+	EmpathyContact        *sender;
 
 	priv = GET_PRIV (window);
 
 	has_focus = empathy_chat_window_has_focus (window);
 
+	/* - if we're the sender, we play the sound if it's specified in the
+	 *   preferences and we're not away.
+	 * - if we receive a message, we play the sound if it's specified in the
+	 *   prefereces and the window does not have focus on the chat receiving
+	 *   the message.
+	 */
+
+	sender = empathy_message_get_sender (message);
+
+	if (empathy_contact_is_user (sender) != FALSE) {
+		empathy_sound_play (GTK_WIDGET (priv->dialog),
+				    EMPATHY_SOUND_MESSAGE_OUTGOING);
+	} else {
+		if ((!has_focus || priv->current_chat != chat)) {
+			empathy_sound_play (GTK_WIDGET (priv->dialog),
+					    EMPATHY_SOUND_MESSAGE_INCOMING);
+		}
+	}
+
+	if (!has_focus) {
+		chat_window_show_or_update_notification (window, message, chat);
+	}
+
 	if (has_focus && priv->current_chat == chat) {
 		return;
 	}
@@ -1058,7 +1168,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		chat = empathy_chat_window_find_chat (account, strv[1]);
 
 		if (!chat) {
-			empathy_dispatcher_chat_with_contact_id (account, strv[2]);
+			empathy_dispatcher_chat_with_contact_id (account, strv[2], NULL, NULL);
 			g_object_unref (account);
 			g_strfreev (strv);
 			return;
@@ -1140,6 +1250,12 @@ chat_window_finalize (GObject *object)
 		g_source_remove (priv->save_geometry_id);
 	}
 
+	if (priv->notification != NULL) {
+		notify_notification_close (priv->notification, NULL);
+		g_object_unref (priv->notification);
+		priv->notification = NULL;
+	}
+
 	chat_windows = g_list_remove (chat_windows, window);
 	gtk_widget_destroy (priv->dialog);
 
@@ -1229,7 +1345,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 
 	g_object_unref (glade);
 
-	priv->chatroom_manager = empathy_chatroom_manager_new (NULL);
+	priv->chatroom_manager = empathy_chatroom_manager_dup_singleton (NULL);
 
 	priv->notebook = gtk_notebook_new ();
  	gtk_notebook_set_group (GTK_NOTEBOOK (priv->notebook), "EmpathyChatWindow"); 
@@ -1254,7 +1370,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 	g_object_unref (accel_group);
 
 	/* Set up smiley menu */
-	smiley_manager = empathy_smiley_manager_new ();
+	smiley_manager = empathy_smiley_manager_dup_singleton ();
 	menu = empathy_smiley_menu_new (smiley_manager,
 					chat_window_insert_smiley_activate_cb,
 					window);
@@ -1533,7 +1649,7 @@ empathy_chat_window_find_chat (McAccount   *account,
 	GList *l;
 
 	g_return_val_if_fail (MC_IS_ACCOUNT (account), NULL);
-	g_return_val_if_fail (!G_STR_EMPTY (id), NULL);
+	g_return_val_if_fail (!EMP_STR_EMPTY (id), NULL);
 
 	for (l = chat_windows; l; l = l->next) {
 		EmpathyChatWindowPriv *priv;
